package ads

import (
	"context"
	"encoding/json"
	"net/http"
	"net/url"
	"time"
)

type SnoozeNextAdResponse struct {
	// A list that contains information about the channel’s snoozes and next upcoming ad after successfully snoozing.
	Data []SnoozeNextAdData `json:"data"`
}

type SnoozeNextAdData struct {
	// The number of snoozes available for the broadcaster.
	SnoozeCount int `json:"snooze_count"`

	// The UTC timestamp when the broadcaster will gain an additional snooze, in RFC3339 format.
	SnoozeRefreshAt time.Time `json:"snooze_refresh_at"`

	// The UTC timestamp of the broadcaster’s next scheduled ad, in RFC3339 format.
	NextAdAt time.Time `json:"next_ad_at"`
}

// If available, pushes back the timestamp of the upcoming automatic mid-roll ad by 5 minutes.
// This endpoint duplicates the snooze functionality in the creator dashboard’s Ads Manager.
//
// Requires a user access token that includes the channel:manage:ads scope.
// The user_id in the user access token must match the broadcaster_id.
func (e *Ads) SnoozeNextAd(ctx context.Context, broadcasterID string) (*SnoozeNextAdResponse, error) {
	endpoint := e.baseUrl.ResolveReference(&url.URL{Path: "channels/ads/schedule/snooze", RawQuery: "broadcaster_id=" + broadcasterID})

	req, err := http.NewRequest(http.MethodPost, endpoint.String(), nil)
	if err != nil {
		return nil, err
	}

	res, err := e.client.Do(req)
	if err != nil {
		return nil, err
	}

	defer res.Body.Close()

	var data SnoozeNextAdResponse
	if err := json.NewDecoder(res.Body).Decode(&data); err != nil {
		return nil, err
	}

	return &data, nil
}
